<?php

/**
 * Copyright (c) D3 Data Development (Inh. Thomas Dartsch)
 *
 * For the full copyright and license information, please view
 * the LICENSE file that was distributed with this source code.
 *
 * https://www.d3data.de
 *
 * @copyright (C) D3 Data Development (Inh. Thomas Dartsch)
 * @author    D3 Data Development - Daniel Seifert <info@shopmodule.com>
 * @link      https://www.oxidmodule.com
 */

declare(strict_types=1);

namespace D3\LoggerFactory;

use Monolog\Processor\ProcessorInterface;

class SensitiveFilterProcessor implements ProcessorInterface
{
    protected string $replacement;

    /**
     * @param string[]       $secrets
     * @param string|null $replacement
     */
    public function __construct(protected array $secrets, ?string $replacement = null)
    {
        $this->replacement = $replacement ?? '*****';
        $this->convertStringsToRegex($this->secrets);
    }

    /**
     * @param string[] $search
     *
     * @return void
     */
    protected function convertStringsToRegex(array $search = []): void
    {
        $searchStrings = [];
        array_map(
            function ($search) use (&$searchStrings) {
                if (!$this->stringIsRegexp($search)) {
                    $searchStrings[] = '/'.preg_quote($search, '/').'/i';
                    if (urlencode($search) !== $search) {
                        $searchStrings[] = '/' . preg_quote(urlencode($search), '/') . '/i';
                    }
                } else {
                    $searchStrings[] = $search;
                }
            },
            $search
        );

        $this->secrets = $searchStrings;
    }

    protected function stringIsRegexp(string $string): bool
    {
        return @preg_match($string, '') !== false;
    }

    public function __invoke(array $records): array
    {
        foreach ($records as $key => $item) {
            if (is_string($item)) {
                $records[$key] = preg_replace(
                    $this->secrets,
                    $this->replacement,
                    $item
                );
            } elseif (is_array($item)) {
                /**  @phpstan-ignore argument.type */
                $records[$key] = $this($item);
            }
        }

        /**  @phpstan-ignore return.type */
        return $records;
    }
}
